<?php
// $Id: product.inc,v 1.1.2.18 2008/01/15 21:05:54 brmassa Exp $

/**
 * @file product.inc
 * Non crucial functions
 * @author e-Commerce Dev Team
 */

/**
 * List all features using some filter parameters
 */
function product_feature_get($op = 'types', $ftype = NULL, $reset = FALSE) {
  static $info, $names;

  if (!empty($reset) || !isset($info)) {
    $info = module_invoke_all('feature_info');
    if (!empty($info)) {
      foreach ($info as $key => $value) {
        $value = (object)$value;
        $value->ftype = $key;
        $names[$key] = $value->name;
        $info[$key] = $value;
      }
      asort($names);
    }
  }

  if (!empty($info) and !empty($ftype) and !isset($info[$ftype])) {
    return FALSE;
  }

  switch ($op) {
    case 'types':
      return $info;
    case 'type':
      return  !empty($info[$ftype]) ? $info[$ftype] : NULL;
    case 'module':
      return $info[$ftype]->module;
    case 'names':
      return $names;
    case 'name':
      return $names[$ftype];
  }
}

/**
 * Get all the functions that are called a node
 *
 * @param $node
 *  Object. product node that is being used
 * @param $hook
 *  String. Hook that is to be called.
 * @return
 *  Array. Hooks that are the functions of all the features.
 */
function product_feature_get_all_functions(&$node, $hook) {
  $funcs = array();
  if ($features = product_ptypes_get('features', $node)) {
    include_once drupal_get_path('module', 'product') .'/product.inc';

    foreach ($features as $ftype => $feature) {
      if ($func = product_feature_get_function($ftype, $node, $hook)) {
        $funcs[] = $func;
      }
    }
  }
  return $funcs;
}

/**
 * Get the function that is called.
 *
 * @param $ftype
 *  Feature type to be called.
 *
 * @param $node
 *  Product node that is being called
 *
 * @param $hook
 *  Hook to called
 *
 * @return
 *  Return the function that is to be called or NULL if no function exists
 */
function product_feature_get_function($ftype, &$node, $hook) {
  include_once drupal_get_path('module', 'product') .'/product.inc';

  if ($module = product_feature_get('module', $ftype)) {
    $function = "{$module}_{$hook}";
    if (function_exists($function)) {
      return $function;
    }
  }
}

/**
 * Returns a price normalized for the price database fields in e-Commerce. The
 * price is normalized to a float format and that value is returned. If $price
 * contains characters other than a leading payment_symbol, decimal point or
 * thousands separator this function returns FALSE.
 *
 * @param $price String representation of the price.
 * @return Normalized price string (as a float) or FALSE if $price contains
 * invalid characters. If $price is an empty string it is returned without
 * flagging it as an error.
 */
function product_price_normalize($price) {
  $price = trim($price);

  if (!empty($price)) {
    $symb = variable_get('payment_symbol', '$');
    $symblen = strlen($symb);
    $symbpos = (variable_get('payment_symbol_position', 1) == 1 ? 0 : -$symblen);
    $pricestart = ($symbpos == 0 ? $symblen : 0);
    $decimal = variable_get('payment_decimal', '.');

    if (substr($price, $symbpos, $symblen) == $symb) {
      $price = substr($price, $pricestart, strlen($price) - $symblen);
    }

    $price = str_replace(variable_get('payment_thousands', ','), '', $price);
    $price = str_replace(variable_get('payment_decimal', '.'), '.', $price);

    if (is_numeric($price)) {
      return $price;
    }
  }
  else {
    return '';
  }

  return FALSE;
}

/**
 * Check if the user can create/edit any kind of product type
 *
 * @param $ptype
 *   Optional name of product type.
 * @param $op
 *   Optional string defining the operation to check.
 * @return
 *   Boolean, TRUE if user can edit at least one of the product types.
 */
function product_ptypes_access($ptype = NULL, $op = NULL) {
  if (empty($ptype)) {
    $ptypes = product_ptypes_get('names');
    while (list(, $ptype) = each($ptypes)) {
      if ($access = user_access('create '. $ptype)) {
        return TRUE;
      }
    }
    return FALSE;
  }
  else {
    if (empty($op) or $op != 'view') {
      return user_access('create '. $ptype);
    }
    return TRUE;
  }
}

/**
 * Builds a list of available node types, by quering hook_product_info() in
 * all modules.
 *
 * @TODO: this function is quite odd, if you understand it, please document it!
 */
function product_ptypes_build() {
  $ptypes = array();
  $names = array();

  $info_array = module_invoke_all('product_info');
  foreach ($info_array as $type => $info) {
    $ptypes[$type] = (object)$info;
    $ptypes[$type]->ptype = $type;
    $ptypes[$type]->internal = TRUE;
    $ptypes[$type]->features = array();
    $names[$type] = $info['name'];
  }

  $result = db_query('select * FROM {ec_product_ptypes}');
  while ($type = db_fetch_object($result)) {
    if (!empty($ptypes[$type->ptype])) {
      if ($type->name) {
        $ptypes[$type->ptype]->name = $type->name;
      }
      if ($type->description) {
        $ptypes[$type->ptype]->description = $type->description;
      }
      $ptypes[$type->ptype]->internal = FALSE;
      $names[$type->ptype] = $ptypes[$type->ptype]->name;
    }
    else {
      $type->module = 'product_generic';
      $type->features = array();
      $ptypes[$type->ptype] = $type;
      $names[$type->ptype] = $type->name;
    }
  }

  include_once drupal_get_path('module', 'product') .'/product.inc';

  $features = product_feature_get();
  $result = db_query('SELECT * FROM {ec_product_features} ORDER BY weight ASC');
  while ($data = db_fetch_object($result)) {
    if (isset($ptypes[$data->ptype])) {
      if (isset($features[$data->ftype])) {
        $feature = $features[$data->ftype];
        $feature->weight = $data->weight;
        $ptypes[$data->ptype]->features[$feature->ftype] = $feature;
      }
    }
  }

  asort($names);
  return array($ptypes, $names);
}

/**
 * Delete a product type
 */
function product_ptypes_delete($ptype) {
  $info = product_ptypes_get('type', $ptype, TRUE);
  db_query("DELETE FROM {ec_product_ptypes} WHERE ptype = '%s'", $ptype);
  module_invoke_all('product_types', $info, 'delete');
}

/**
 * Save a product type
 */
function product_ptypes_save($info) {
  $ptype = isset($info->old_ptype) ? $info->old_ptype : $info->ptype;
  if (db_result(db_query_range("SELECT COUNT(*) FROM {ec_product_ptypes}
      WHERE ptype = '%s'", $ptype, 0, 1))) {
    db_query("UPDATE {ec_product_ptypes} SET name = '%s', ptype = '%s',
      description = '%s' WHERE ptype = '%s'", $info->name, $info->ptype,
      $info->description, $ptype);
  }
  else {
    db_query("INSERT INTO {ec_product_ptypes} (name, ptype, description)
      VALUES ('%s', '%s', '%s')", $info->name, $info->ptype, $info->description);
  }
  $info = product_ptypes_get('type', $info->ptype, TRUE);
  module_invoke_all('product_types', $info, 'update');
}

/**
 * List all price adjustments for a product
 *
 * @param $node
 *   Object. The product
 * @param $type
 * @param $total_only
 *   Boolean, TRUE if only the total price is important
 * @param $txn
 */
function product_specials_get($node, $type, $total_only = FALSE, $txn = NULL) {
  $specials = array();
  if (!(product_attributes_get($node, 'no_discounts'))) {
    foreach (module_implements('product_specials') as $name) {
      if ($extra = module_invoke($name, 'product_specials', $node, $specials, $type, $txn)) {
        $specials = array_merge($specials, $extra);
      }
    }
  }
  if (!empty($total_only)) {
    $total = 0;
    foreach ($specials as $special) {
      if (is_array($special)) {
        $total+= $special['price']*(isset($special['qty']) ? $special['qty'] :
          (isset($node->qty) ? $node->qty : 1));
      }
      else {
        $total+= $special*(isset($node->qty) ? $node->qty : 1);
      }
    }
    return $total;
  }
  else {
    return $specials;
  }
}

/**
 * Display the product price
 *
 * @param $node
 *   Object. The node
 * @ingroup themeable
 */
function theme_product_price($node) {
  include_once drupal_get_path('module', 'product') .'/product.inc';

  // Add the CSS
  drupal_add_css(drupal_get_path('module', 'product') .'/product.css');

  return '<div class="price">'.
    format_currency(product_price_adjust($node, 'product') +
    product_specials_get($node, 'product', TRUE)) .'</div>';
}
